@@ -1,6 +1,6 @@
 use super::OpTrait;
 use crate::{
-    git::diff::{Hunk, PatchMode},
+    git::diff::{Diff, PatchMode},
     items::TargetData,
     state::State,
     term::Term,
@@ -15,9 +15,20 @@ impl OpTrait for Stage {
             Some(TargetData::AllUnstaged) => stage_unstaged(),
             Some(TargetData::AllUntracked(untracked)) => stage_untracked(untracked),
             Some(TargetData::File(u)) => stage_file(u.into()),
-            Some(TargetData::Delta(d)) => stage_file(d.new_file.into()),
-            Some(TargetData::Hunk(h)) => stage_patch(h),
-            Some(TargetData::HunkLine(h, i)) => stage_line(h, i),
+            Some(TargetData::Delta { diff, file_i }) => {
+                stage_file(diff.text[diff.file_diffs[file_i].header.new_file.clone()].into())
+            }
+            Some(TargetData::Hunk {
+                diff,
+                file_i,
+                hunk_i,
+            }) => stage_patch(diff, file_i, hunk_i),
+            Some(TargetData::HunkLine {
+                diff,
+                file_i,
+                hunk_i,
+                line_i,
+            }) => stage_line(diff, file_i, hunk_i, line_i),
             _ => return None,
         };
 
@@ -65,23 +76,23 @@ fn stage_file(file: OsString) -> Action {
     })
 }
 
-fn stage_patch(h: Rc<Hunk>) -> Action {
+fn stage_patch(diff: Rc<Diff>, file_i: usize, hunk_i: usize) -> Action {
     Rc::new(move |state, term| {
         let mut cmd = Command::new("git");
         cmd.args(["apply", "--cached"]);
 
         state.close_menu();
-        state.run_cmd(term, &h.format_patch().into_bytes(), cmd)
+        state.run_cmd(term, &diff.format_patch(file_i, hunk_i).into_bytes(), cmd)
     })
 }
 
-fn stage_line(h: Rc<Hunk>, i: usize) -> Action {
+fn stage_line(diff: Rc<Diff>, file_i: usize, hunk_i: usize, line_i: usize) -> Action {
     Rc::new(move |state, term| {
         let mut cmd = Command::new("git");
         cmd.args(["apply", "--cached", "--recount"]);
 
-        let input = h
-            .format_line_patch(i..(i + 1), PatchMode::Normal)
+        let input = diff
+            .format_line_patch(file_i, hunk_i, line_i..(line_i + 1), PatchMode::Normal)
             .into_bytes();
 
         state.close_menu();
