@@ -17,6 +17,7 @@ use std::fs;
 #[macro_use]
 mod helpers;
 mod arg;
+mod branch;
 mod commit;
 mod discard;
 mod editor;
@@ -257,36 +258,6 @@ mod show_refs {
     }
 }
 
-mod checkout {
-    use super::*;
-
-    #[test]
-    pub(crate) fn checkout_menu() {
-        let ctx = TestContext::setup_clone();
-        run(ctx.dir.path(), &["git", "branch", "other-branch"]);
-        snapshot!(ctx, "Yjb");
-    }
-
-    #[test]
-    pub(crate) fn switch_branch_selected() {
-        let ctx = TestContext::setup_clone();
-        run(ctx.dir.path(), &["git", "branch", "other-branch"]);
-        snapshot!(ctx, "Yjjbb<enter>");
-    }
-
-    #[test]
-    pub(crate) fn switch_branch_input() {
-        let ctx = TestContext::setup_clone();
-        run(ctx.dir.path(), &["git", "branch", "hi"]);
-        snapshot!(ctx, "Yjjbbhi<enter>");
-    }
-
-    #[test]
-    pub(crate) fn checkout_new_branch() {
-        snapshot!(TestContext::setup_clone(), "bcf<esc>cx<enter>");
-    }
-}
-
 #[test]
 fn updated_externally() {
     let mut ctx = TestContext::setup_init();
@@ -389,3 +360,21 @@ fn tab_diff() {
 
     insta::assert_snapshot!(ctx.redact_buffer());
 }
+
+#[test]
+fn ext_diff() {
+    let mut ctx = TestContext::setup_init();
+    let mut state = ctx.init_state();
+
+    fs::write(ctx.dir.child("unstaged.txt"), "unstaged\n").unwrap();
+    fs::write(ctx.dir.child("staged.txt"), "staged\n").unwrap();
+    run(ctx.dir.path(), &["git", "add", "-N", "unstaged.txt"]);
+    run(ctx.dir.path(), &["git", "add", "staged.txt"]);
+    run(
+        ctx.dir.path(),
+        &["git", "config", "diff.external", "/dev/null"],
+    );
+    ctx.update(&mut state, keys("g"));
+
+    insta::assert_snapshot!(ctx.redact_buffer());
+}
